@@ -1089,13 +1089,7 @@ xfce_settings_editor_box_channel_menu (XfceSettingsEditorBox *self)
         G_CALLBACK (xfce_settings_editor_box_load_channels), self);
     gtk_widget_show (mi);
 
-#if GTK_CHECK_VERSION (3, 22, 0)
     gtk_menu_popup_at_pointer (GTK_MENU (menu), NULL);
-#else
-    gtk_menu_popup (GTK_MENU (menu),
-                    NULL, NULL, NULL, NULL, 3,
-                    gtk_get_current_event_time ());
-#endif
 
     return TRUE;
 }
